"""
This package contains description of the function
that allows to initialize custom classes with clear
and informative `repr` method and base class with such method.

"""


from .main import get_representation, RepresentableObject
